#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Operators;

namespace HeuristicLab.Evolutionary {
  public abstract class CrossoverBase : OperatorBase {
    public CrossoverBase()
      : base() {
      AddVariableInfo(new VariableInfo("Random", "Pseudo random number generator", typeof(IRandom), VariableKind.In));
    }

    public override IOperation Apply(IScope scope) {
      IRandom random = GetVariableValue<IRandom>("Random", scope, true);

      if ((scope.SubScopes.Count % 2) != 0)
        throw new InvalidOperationException("Size of mating pool is not even");

      int children = scope.SubScopes.Count / 2;
      for (int i = 0; i < children; i++) {
        IScope parent1 = scope.SubScopes[0];
        IScope parent2 = scope.SubScopes[1];
        IScope child = new Scope(i.ToString());
        scope.AddSubScope(child);
        Cross(scope, random, parent1, parent2, child);
        scope.RemoveSubScope(parent1);
        scope.RemoveSubScope(parent2);
      }

      return null;
    }

    protected abstract void Cross(IScope scope, IRandom random, IScope parent1, IScope parent2, IScope child);
  }
}
